package anywheresoftware.b4a.jalle007.wificonnect;

import java.io.BufferedInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.net.InetAddress;
import java.util.List;

import android.content.Context;
import android.net.wifi.ScanResult;
import android.net.wifi.WifiConfiguration;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import android.net.wifi.WpsInfo;
import android.text.format.Formatter;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.Author;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.BA.Version;

@Author("jalle007")
@Version(1.59f)
@ShortName("WiFiConnect")

public class WiFiConnect
{
	//static int netId;
	
	// 1.0 initial release
	// 1.1 added test for no networks found and added IsEnabled 
	// 1.2 22/03/2015 edited from Massimo Meli : added javadocs and these methods: 
	               // DisconnectFromSSID
	               // getWifiState
	               // getConfiguredNetworks
	               // setWifiEnabled
	               // getWifiEnabled
	// 1.3 16/05/2015: Added new method "SwitchSSID"
	// 1.59 26/05/2015: Added new methods "getSSID" & "updateInfo"
	
	private final static double version = 1.59;
	
	/**
	 *Returns the version number of the library.
	 */
	public double getVersion()
	{
		return version;
	}
	
	//Some network info constants
	public static final String LINK_SPEED_UNITS = "Mbps";
	public static final String FREQUENCY_UNITS = "MHz";
	
	//WiFi Encrypt
	public static final int WIFI_OPEN = 0;
	public static final int WIFI_WEP = 1;
	public static final int WIFI_WPA = 2;

	//WiFi State
	public static final int WIFI_STATE_UNKNOWN = -1;
	public static final int WIFI_STATE_DISABLING = 0;
	public static final int WIFI_STATE_DISABLED = 1;
	public static final int WIFI_STATE_ENABLING = 2;
	public static final int WIFI_STATE_ENABLED = 3;
	public static final int WIFI_STATE_FAILED = 4;
	
	//WiFi Configurations status
	public static final int CURRENT = 0; // this is the network we are currently connected to
	public static final int DISABLED = 1; // supplicant will not attempt to use this network
	public static final int ENABLED = 2; // supplicant will consider this network available for association
	
	public static final int WIFI_MODE_FULL = 1;
	public static final int WIFI_MODE_SCAN_ONLY = 2;
	public static final int WIFI_MODE_FULL_HIGH_PERF = 3;

	private static int Delay = 0;
	
	private static WifiManager wifiManager; 
	private static WifiInfo info;
	
	private static boolean inizialized = false;
	
	/**
	 * Initialize this class.
	 */
	public static void Initialize(BA ba) throws Exception
   {
		wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
		info = wifiManager.getConnectionInfo();
		inizialized = true;
	}
	
	/**
	 * Check if class is initialized
	 */
	public static boolean isInitialized(BA ba) throws Exception
	{
		return inizialized;
	}
	
	/**
	 * QUESTO LUCA L'HO MESSO PER AGGIUNGERE DEI DELAYS TRA
	 * LE VARIE LETTURE NEI DEVICES VECCHI, POI OGNI METODO METTO
	 * UN  Thread.sleep(DelayTime) MA PURTROPPO ANCHE COS
	 * LE INFOS NON VENGONO LETTE SUL MIO TABLET CON aNDROID 2.3.1	
	 * QUINDI PENSO CHE LO TOGLIER
	 * 
	 * Set or Get the delay time between read infos, these only needed for old 
	 * devices compatibility as these devices require more time to read network infos.
	 * By default if this method is not called, delay time is 0
	 * 
	 * DelayTime: the delay time (milliseconds) between each read info
	 */
	public void setDelayTime(int DelayTime)
	{
		Delay	= DelayTime;
	}
	
	public int getDelayTime()
	{
		return Delay;
	}
	
	/**
	 * Connect to a specific SSID.
	 * 
	 * wifi_security: Security type (use constants)
	 * SSID: The SSID of the network to which we want to connect
	 * Pass: The password of the network to which we want to connect
	 */
	public boolean ConnectToSSID(BA ba, int wifi_security, String SSID, String Pass) throws Exception
	{	
//		WifiManager wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
		
		//ADD CONFIGURATION		
		int netid = -1;
		if (getNetworkIdBySSID(ba, SSID) == -1) {
			
			WifiConfiguration conf = new WifiConfiguration();
			conf.SSID = "\"" + SSID + "\""; // Please note the quotes. String should contain ssid in quotes
	                                          
			switch (wifi_security) {
			case 0: // OPEN
				conf.allowedKeyManagement.set(WifiConfiguration.KeyMgmt.NONE);

			case 1: // WEP
				boolean isHex = Pass.matches("^[0-9a-fA-F]+$");
				if (isHex == true) conf.wepKeys[0] = Pass;    //In case of WEP, if your password is in hex, you do not need to surround it with quotes.
				else conf.wepKeys[0] = "\"" + Pass + "\"";    
				conf.wepTxKeyIndex = 0;
				conf.allowedKeyManagement.set(WifiConfiguration.KeyMgmt.NONE);
				conf.allowedGroupCiphers.set(WifiConfiguration.GroupCipher.WEP40);
				break;

			case 2: // WPA
				conf.preSharedKey = "\"" + Pass + "\"";   

			default:
				break;
			}
			
			conf.priority = 999999;
			if (wifiManager.pingSupplicant()) {
				//wifiManager.updateNetwork(conf);
				wifiManager.saveConfiguration();
			}
			netid = wifiManager.addNetwork(conf);
		}
		else {
			netid = getNetworkIdBySSID(ba, SSID);
		}
				
		// CONNECT
		boolean success = false;
		boolean success1 = false;
		boolean success2 = false;
		boolean success3 = false;
		
		List<WifiConfiguration> list = wifiManager.getConfiguredNetworks();
		for (WifiConfiguration i : list)
		{
			//if (i.SSID != null && i.SSID.equals("\"" + networkSSID + "\""))
			if (i.networkId != -1 && i.networkId == netid)
			{
				success1 = wifiManager.disconnect();
				//success2 = wifiManager.enableNetwork(i.networkId, true);
				success2 = wifiManager.enableNetwork(netid, true);
				success3 = wifiManager.reconnect();
				success = success1 && success2 && success3;
				
				//BA.Log("wifiManager.removeNetwork(conf.networkId) - success1: " + success1);  
				//BA.Log("wifiManager.disconnect() - success2: " + success2);
				//BA.Log("wifiManager.disableNetwork(conf.networkId) - success3: " + success3);
				//BA.Log("success1 && success2 && success3: " + success);
				
				break;
			}
		}
				
		return success;
	}
	
	/**
	 * Disconnect from a current connected SSID.
	 */
	//public boolean DisconnectFromSSID(BA ba, String SSID) throws Exception
	public boolean DisconnectCurrentSSID(BA ba) throws Exception
	{	
		boolean success = false;
		boolean success1 = false;
		boolean success2 = false;
		
//		WifiConfiguration conf = new WifiConfiguration();    //// see this line
	
//		WifiManager wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
//		conf.SSID = "\"" + networkSSID + "\""; // Please note the quotes. String should contain ssid in quotes
		
		List<WifiConfiguration> list = wifiManager.getConfiguredNetworks();
		for (WifiConfiguration i : list)
			{
				if (i.SSID != null)
				{
					//success1 = wifiManager.removeNetwork(conf.networkId);
					success1 = wifiManager.disconnect();
					success2 = wifiManager.disableNetwork(i.networkId);
				}
			}
		
		success = success1 && success2;
		return success;	
	}
	
	
	/**
	 * Switch from a specific SSID to another SSID
	 * 
	 * SSID: The target network SSID which we want to connect
	 * wifi_security: Security type (use constants)
	 * Pass: The password of the network to which we want to connect
	 */
	public boolean SwitchSSID(BA ba, String SSID, int wifi_security, String networkPass) throws Exception
	{			
		DisconnectCurrentSSID(ba);
		//try to connected to the target SSID
		boolean success = ConnectToSSID(ba, wifi_security, SSID, networkPass);
		return success;
	}
	
	/**
	 * Return the NetworkID associated with a specific SSID
	 * 
	 * SSID: pass the WiFi SSID
	 */
	public int getNetworkIdBySSID(BA ba, String SSID) throws Exception
	{
//		WifiManager wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
		List<WifiConfiguration> list = wifiManager.getConfiguredNetworks();
		
		int NetId = -1;
		for (WifiConfiguration i : list)
		{
			if (i.SSID != null && i.SSID.equals("\"" + SSID + "\""))
			{
				NetId = i.networkId;
				break;
			}
		}
		return NetId;
	}
	
	// WifiInfo **********************************
	
	/**
	 * Return the MAC Address of current active connection.
	 */
	public String getMacAddress(BA ba) throws Exception
	{
//		WifiManager wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
		String MacAddress;
		
		if(wifiManager.isWifiEnabled()) {
		    // WIFI ALREADY ENABLED. GRAB THE MAC ADDRESS HERE
//		    WifiInfo info = wifiManager.getConnectionInfo();
//			 info = wifiManager.getConnectionInfo();
		    MacAddress = info.getMacAddress();
		} else {
		    // ENABLE THE WIFI FIRST
		    wifiManager.setWifiEnabled(true);
	
		    // WIFI IS NOW ENABLED. GRAB THE MAC ADDRESS HERE
//		    WifiInfo info = wifiManager.getConnectionInfo();
//		    info = wifiManager.getConnectionInfo();
		    MacAddress = info.getMacAddress();
		    
		    // DISABLE THE WIFI 
		    wifiManager.setWifiEnabled(true);
		}
		return MacAddress;
	}
	
	/**
	 * Calculates the absolute signal level of the signal. This should be used any time a signal is being shown.
	 *
	 * numLevels: The number of levels to consider in the calculated level.
	 */
	public int getSignalLevel(BA ba, int numLevels)throws Exception
	{
//		WifiManager wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
//		WifiInfo info = wifiManager.getConnectionInfo();
//		info = wifiManager.getConnectionInfo();
		return  WifiManager.calculateSignalLevel(info.getRssi(),numLevels);
	}
	
	/**
	 * Returns the received signal strength indicator of the current 802.11 network, in dBm.
	 */
	public int getRssi(BA ba)throws Exception
	{
//		WifiManager wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
//		WifiInfo info = wifiManager.getConnectionInfo();
//		info = wifiManager.getConnectionInfo();
		return  info.getRssi();
	}
	
	/**
	 * Return the basic service set identifier (BSSID) of the current access point.
	 * The BSSID may be null if there is no network currently connected.
	 * 
	 * the BSSID is in the form of a six byte MAC address: XX:XX:XX:XX:XX:XX
	 */
	public String getBSSID(BA ba)throws Exception
	{	
//		WifiManager wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
//		WifiInfo info = wifiManager.getConnectionInfo();
//		info = wifiManager.getConnectionInfo();
		return  info.getBSSID();
	}
	
	/**
	 * Returns the current frequency in MHz.
	 */
	public int getFrequency(BA ba)throws Exception
	{
//		WifiManager wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
//		WifiInfo info = wifiManager.getConnectionInfo();
//		info = wifiManager.getConnectionInfo();	
		return  info.getFrequency();
	}
	
	/**
	 * Returns the current link speed in Mbps.
	 */
	public int getLinkSpeed(BA ba)throws Exception
	{
//		WifiManager wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
//		WifiInfo info = wifiManager.getConnectionInfo();
//		info = wifiManager.getConnectionInfo();	
		return  info.getLinkSpeed();
	}
	
	/**
	 * Returns True if this network does not broadcast its SSID.
	 */
	public boolean getHiddenSSID(BA ba)throws Exception
	{
//		WifiManager wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
//		WifiInfo info = wifiManager.getConnectionInfo();
//		info = wifiManager.getConnectionInfo();
		return  info.getHiddenSSID();
	}
	
	/**
	 * Returns the SSID of the current connected network
	 */
	public String getSSID(BA ba)throws Exception
	{
		return  info.getSSID();
	}
	
	/**
	 * Returns IP address of device as integer.
	 */
	public int getIpAddressAsInt(BA ba)throws Exception
	{
//		WifiManager wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
//		WifiInfo info = wifiManager.getConnectionInfo();
//		info = wifiManager.getConnectionInfo();
		return  info.getIpAddress();
	}
	
	/**
	 * Returns IP address of device as string.
	 * 
	 * formatIpAddress is deprecated, but for now we use this. 
	 */
	@SuppressWarnings("deprecation")
	public String getIpAddress(BA ba)throws Exception
	{
//		WifiManager wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
		//WifiInfo info = wifiManager.getConnectionInfo();
		//return  info.getIpAddress(); 
		return Formatter.formatIpAddress(wifiManager.getConnectionInfo().getIpAddress());
	} 
	
   // WifiManager **********************************
	
	/**
	 * Add a new network description to the set of configured networks. The networkId field of the 
	 * supplied configuration object is ignored.
	 * 
	 * The new network will be marked DISABLED by default. To enable it, called enableNetwork(int, boolean).
	 * 
	 * config: the set of variables that describe the configuration, contained in a WifiConfiguration object.
	 * 
	 * Returns the ID of the newly created network description. This is used in other operations to specified 
	 * the network to be acted upon. Returns -1 on failure.
	 */
	public int addNetwork(BA ba, WifiConfiguration config) throws Exception
	{
//		WifiManager wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
		return wifiManager.addNetwork(config);				
	}
	
	/**
	 * Disassociate from the currently active access point. This may result in the asynchronous 
	 * delivery of state change events.
	 * 
	 * Returns true if the operation succeeded
	 */
	public boolean disconnect(BA ba) throws Exception
	{
//		WifiManager wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
		return wifiManager.disconnect();		
	}
	
	/**
	 * Allow a previously configured network to be associated with. If disableOthers is true, 
	 * then all other configured networks are disabled, and an attempt to connect to the selected 
	 * network is initiated. This may result in the asynchronous delivery of state change events.
	 * 
	 * netId:  the ID of the network in the list of configured networks
	 * disableOthers:  if true, disable all other networks. The way to select a particular network 
	 * to connect to is specify true for this parameter.
	 * 
	 * Return True if the operation succeeded
	 */
	public boolean enableNetwork(BA ba, int netId, boolean disableOthers) throws Exception
	{
//		WifiManager wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
		return wifiManager.enableNetwork(netId, disableOthers);			
	}
	
	/**
	 * Disable a configured network. The specified network will not be a candidate for associating. 
	 * This may result in the asynchronous delivery of state change events.
	 * 
	 * netId: the ID of the network as returned by addNetwork(WifiConfiguration).
	 * 
	 * Return True if the operation succeeded
	 */
	public boolean disableNetwork(BA ba, int netId) throws Exception
	{
//		WifiManager wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
		return wifiManager.disableNetwork(netId);					
	}
	
	/** 
	 * Gets a list with configured networks.
	 */
	public List<WifiConfiguration> getConfiguredNetworks(BA ba) throws Exception
	{
//		WifiManager wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
		return wifiManager.getConfiguredNetworks();			
	}
	
	/**
	 * Calculates the level of the signal. This should be used any time a signal is being shown.
	 * 
	 * rssi: The power of the signal measured in RSSI.
	 * numLevels:  The number of levels to consider in the calculated level.
	 * 
	 * Returns a level of the signal, given in the range of 0 to numLevels-1 (both inclusive).
	 */
	public static int calculateSignalLevel(BA ba, int rssi, int numLevels) throws Exception
	{
		return WifiManager.calculateSignalLevel(rssi, numLevels);				
	}
	
	/**
	 * Compares two signal strengths.
	 * 
	 * rssiA:  The power of the first signal measured in RSSI.
	 * rssiB:  The power of the second signal measured in RSSI.
	 * 
	 * Returns < 0 if the first signal is weaker than the second signal, 0 if the two signals have 
	 * the same strength, and > 0 if the first signal is stronger than the second signal.
	 */
	public static int compareSignalLevel(BA ba, int rssiA, int rssiB) throws Exception
	{
		return WifiManager.compareSignalLevel(rssiA, rssiB);				
	}
	
	/**
	 * Create a new MulticastLock.
	 * 
	 * tag: a tag for the MulticastLock to identify it in debugging messages. This string is never 
	 * shown to the user under normal conditions, but should be descriptive enough to identify your 
	 * application and the specific MulticastLock within it, if it holds multiple MulticastLocks.
	 * 
	 * Returns a new, unacquired MulticastLock with the given tag.
	 */
	public WifiManager.MulticastLock createMulticastLock(BA ba, String tag) throws Exception
	{
//		WifiManager wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
		return wifiManager.createMulticastLock(tag);			
	}
	
	/**
	 * Creates a new WifiLock.
	 * 
	 * tag: a tag for the WifiLock to identify it in debugging messages. This string is never shown 
	 * to the user under normal conditions, but should be descriptive enough to identify your 
	 * application and the specific WifiLock within it, if it holds multiple WifiLocks.
	 * 
	 * Returns a new, unacquired WifiLock with the given tag.
	 * @throws Exception
	 */
	public WifiManager.WifiLock createWifiLock1(BA ba, String tag) throws Exception
	{
//		WifiManager wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
		return wifiManager.createWifiLock(tag);			
	}
	
	/**
	 * Creates a new WifiLock.
	 * 
	 * lockType: the type of lock to create. See WIFI_MODE_FULL, WIFI_MODE_FULL_HIGH_PERF and WIFI_MODE_SCAN_ONLY for descriptions of the types of WiFi locks. 
	 * tag: a tag for the WifiLock to identify it in debugging messages. This string is never shown to the user under normal conditions, but should be 
	 * descriptive enough to identify your application and the specific WifiLock within it, if it holds multiple WifiLocks.
	 * 
	 * Returns a new, unacquired WifiLock with the given tag.

	 */
	public WifiManager.WifiLock createWifiLock2(BA ba, int lockType, String tag) throws Exception
	{
//		WifiManager wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
		return wifiManager.createWifiLock(lockType, tag);			
	}
	
	/**
	 * Cancel any ongoing Wifi Protected Setup.
	 * 
	 * listener: for callbacks on success or failure. Can be null.
	 * 
	 * Throws IllegalStateException	if the WifiManager instance needs to be initialized again
	 */
	public void cancelWps(BA ba, WifiManager.WpsCallback listener) throws IllegalStateException
	{
//		WifiManager wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
		wifiManager.cancelWps(listener);;		
	}
	
   /**
    * Sets the WiFi Enabled state.
    * 
    * enabled: set WiFi Enabled (set True to enable or False do disable)
    */
	public void setWifiEnabled(BA ba, boolean enabled)
		{		
//			WifiManager wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
			wifiManager.setWifiEnabled(enabled);
		}
	
   /**
    * Gets the WiFi Enabled state.
    * 
    * return True if enabled or False if disabled
    */
	public boolean isWifiEnabled(BA ba)throws Exception
	{
//		WifiManager wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
		return wifiManager.isWifiEnabled();
	}	
	
   /**
    * Gets the WiFi State (see costants)
    * 
    * Return the WiFi state
    */
	public int getWifiState(BA ba)
		{		
//			WifiManager wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
			return wifiManager.getWifiState();
		}
	
	/**
	 * Returns True if this adapter supports 5 GHz band.
	 */
	public boolean is5GHzBandSupported(BA ba)throws Exception
	{
//		WifiManager wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
		return wifiManager.is5GHzBandSupported();
	}
	
	/**
	 * Returns True if this adapter supports Device to AP RTT.
	 */
	public boolean isDeviceToApRttSupported(BA ba)throws Exception
	{
//		WifiManager wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
		return wifiManager.isDeviceToApRttSupported();
	}	
	
	/**
	 * Returns True if this adapter supports advanced power/performance counters.
	 */
	public boolean isEnhancedPowerReportingSupported(BA ba)throws Exception
	{
//		WifiManager wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
		return wifiManager.isEnhancedPowerReportingSupported();
	}	
	
	/**
	 * Returns True if this adapter supports WifiP2pManager (WiFi Direct).
	 */
	public boolean isP2pSupported(BA ba)throws Exception
	{
//		WifiManager wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
		return wifiManager.isP2pSupported();
	}	
	
	/**
	 * Returns True if this adapter supports offloaded connectivity scan.
	 */
	public boolean isPreferredNetworkOffloadSupported(BA ba)throws Exception
	{
//		WifiManager wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
		return wifiManager.isPreferredNetworkOffloadSupported();
	}		
	
	/**
	 * Check if scanning is always available. If this return True, apps can issue startScan()
	 * and fetch scan results even when WiFi is turned off.
	 */
	public boolean isScanAlwaysAvailable(BA ba)throws Exception
	{
//		WifiManager wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
		return wifiManager.isScanAlwaysAvailable();
	}		
	
	/**
	 * Returns True if this adapter supports Tunnel Directed Link Setup.
	 */
	public boolean isTdlsSupported(BA ba)throws Exception
	{
//		WifiManager wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
		return wifiManager.isTdlsSupported();
	}		
	
	/**
	 * Check that the supplicant daemon is responding to requests.
	 * Returns True if we were able to communicate with the supplicant 
	 * and it returned the expected response to the PING message.
	 */
	public boolean pingSupplicant(BA ba)throws Exception
	{
//		WifiManager wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
		return wifiManager.pingSupplicant();
	}			

	/**
	 * Tell the supplicant to persist the current list of configured networks.
	 * 
	 * Note: It is possible for this method to change the network IDs of existing networks. 
	 * You should assume the network IDs can be different after calling this method. 
	 * 
	 * Return True if the operation succeeded
	 */
	public boolean saveConfiguration(BA ba)throws Exception
	{
//		WifiManager wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
		return wifiManager.saveConfiguration();
	}				
	
	/**
	 * Enable/Disable TDLS on a specific local route.
	 * 
	 * TDLS enables two wireless endpoints to talk to each other directly without going through
	 * the access point that is managing the local network. It saves bandwidth and improves 
	 * quality of the link. 
	 * 
	 * This API enables/disables the option of using TDLS. If enabled, the underlying hardware
	 * is free to use TDLS or a hop through the access point. If disabled, existing TDLS session
	 * is torn down and hardware is restricted to use access point for transferring wireless packets.
	 * Default value for all routes is 'disabled', meaning restricted to use access point for 
	 * transferring packets.
	 * 
	 * remoteIPAddress:	IP address of the endpoint to setup TDLS with
	 * enable:	True = setup and False = tear down TDLS
	 */
	public void setTdlsEnabled (BA ba, InetAddress remoteIPAddress, boolean enable)throws Exception
	{
//		WifiManager wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
		wifiManager.setTdlsEnabled(remoteIPAddress, enable);;
	}				
	
   /**
    * Similar to setTdlsEnabled(InetAddress, boolean), except this version allows you to specify remote 
    * endpoint with a MAC address.
    * 
    * remoteMacAddress:	MAC address of the remote endpoint such as 00:00:0c:9f:f2:ab
    * enable:	True = setup and False = tear down TDLS
    */
	public void setTdlsEnabledWithMacAddress(BA ba, String remoteMacAddress, boolean enable)throws Exception
	{
//		WifiManager wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
		wifiManager.setTdlsEnabledWithMacAddress(remoteMacAddress, enable);
	}				
	
	/**
	 * Request a scan for access points. Returns immediately. The availability of the results is made known 
	 * later by means of an asynchronous event sent on completion of the scan.
	 * 
	 * Returns True if the operation succeeded, i.e., the scan was initiated
	 */
	public boolean startScan(BA ba) throws Exception
	{
//		WifiManager wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
		return wifiManager.startScan();	
	}
	
	/**
	 * Return the results of the latest access point scan.
	 * 
	 * Returns the list of access points found in the most recent scan.
	 */
	public List<ScanResult> getScanResults(BA ba) throws Exception
	{
//		WifiManager wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
		return wifiManager.getScanResults();		
	}
	
	/**
	 * Start Wifi Protected Setup
	 * 
	 * config:  WPS configuration (does not support LABEL)
	 * listener: for callbacks on success or failure. Can be null.
	 */
	public void startWps (BA ba, WpsInfo config, WifiManager.WpsCallback listener)throws IllegalStateException
	{
//		WifiManager wifiManager = (WifiManager) ba.context.getSystemService(Context.WIFI_SERVICE);
		wifiManager.startWps(config, listener);				
	}
	
	// UTILS **********************************
	
	 /**
    * Convert byte array to hex string
    * 
    * bytes: bytes to convert
    */
   public static String bytesToHex(byte[] bytes) {
       StringBuilder sbuf = new StringBuilder();
       for(int idx=0; idx < bytes.length; idx++) {
           int intVal = bytes[idx] & 0xff;
           if (intVal < 0x10) sbuf.append("0");
           sbuf.append(Integer.toHexString(intVal).toUpperCase());
       }
       return sbuf.toString();
   }
	
   /**
    * Get utf8 byte array.
    * Return  array of NULL if error was found
    * 
    * str: the string to get as bytes
    */
   public static byte[] getUTF8Bytes(String str) {
       try { return str.getBytes("UTF-8"); } catch (Exception ex) { return null; }
   }
	
   /**
    * Load UTF8withBOM or any ansi text file.
    * 
    * filename: the file to load as string
    * 
    * Return a file converted to string
    */
   public static String loadFileAsString(String filename) throws java.io.IOException {
       final int BUFLEN = 1024;
       BufferedInputStream is = new BufferedInputStream(new FileInputStream(filename), BUFLEN);
       try {
           ByteArrayOutputStream baos = new ByteArrayOutputStream(BUFLEN);
           byte[] bytes = new byte[BUFLEN];
           boolean isUTF8 = false;
           int read,count=0;           
           while((read=is.read(bytes)) != -1) {
               if (count==0 && bytes[0]==(byte)0xEF && bytes[1]==(byte)0xBB && bytes[2]==(byte)0xBF ) {
                   isUTF8=true;
                   baos.write(bytes, 3, read-3); // drop UTF8 bom marker
               } else {
                   baos.write(bytes, 0, read);
               }
               count+=read;
           }
           return isUTF8 ? new String(baos.toByteArray(), "UTF-8") : new String(baos.toByteArray());
       } finally {
           try{ is.close(); } catch(Exception ex){} 
       }
   }
   
	/**
	 * Return WiFiInfo
	 */
	public void updateInfo(BA ba) throws Exception
	{
		//update network info
		info = wifiManager.getConnectionInfo();		
	}

}

